﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using WindowsFormsApplication1.WebTest.WebTestXmlNodes.New;

namespace WindowsFormsApplication1.WebTest.WebTestXmlNodes
{
    [XmlRoot("Items", Namespace = XmlNamespace)]
    public class ItemsNode : GenericWebTestNode, IXmlSerializable
    {
        public override void SetNodeProperties()
        {
            SelectedImageIndex = 0;
            ImageIndex = 0;
        }

        #region IXmlSerializable Members

        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(XmlReader reader)
        {
            Notification.BeginOperation("Processing Request Collection Node - <Items>", NodeOperationEventArgs.EventImportance.Detailed);

            //Read Web Test attributes
            ReadXmlAttributes(reader);

            //Set node properties such as Icon, Text, etc..
            SetNodeProperties();

            //Move to the first element
            reader.MoveToContent();
            bool isEmptyElement = reader.IsEmptyElement;

            reader.Read();
            reader.MoveToContent();

            bool exitLoop = false;
            while (!isEmptyElement && reader.NodeType != XmlNodeType.EndElement)
            {
                //Process sub tags
                switch (reader.Name)
                {
                    case "Request":
                        RequestNode request = (RequestNode) _requestNodeSerializer.Deserialize(reader);
                        Nodes.Add(request);
                        break;

                    case "TransactionTimer":
                        TransactionTimerNode transactionTimer = (TransactionTimerNode)_transactionTimerNodeSerializer.Deserialize(reader);
                        Nodes.Add(transactionTimer);
                        break;

                    case "IncludedWebTest":
                        IncludedWebTestNode includedWebTest = (IncludedWebTestNode)_includedWebTestNodeSerializer.Deserialize(reader);
                        Nodes.Add(includedWebTest);
                        break;
                        
                    case "Comment":
                        CommentNode comment = (CommentNode)_commentNodeSerializer.Deserialize(reader);
                        Nodes.Add(comment);
                        break;

                    default:
                        Notification.DebugMessage("Unknown tag: " + reader.Name);
                        exitLoop = true;
                        break;
                }

                reader.MoveToContent();
                if (exitLoop) break;
            }

            //If there is closing tag read it
            if (!isEmptyElement)
                reader.ReadEndElement();

            //reader.MoveToContent();

            Notification.EndOperation();
        }


        #endregion

    }
}
